@@ -18,9 +18,9 @@
 ##< Global Variables >##
 moddir="/modules"   # Kernel module directory
 sysdir="/MOL"       # MOL System directory
-looproot="${sysdir}/MOL" # Loopbak image file
-
-BASE_FLAG="/etc/mol/MOLTYPE"
+looproot="/MOL/MOL" # Loopbak image file
+BASE_FLAG="/MOL/MOLFLAG"
+#BASE_FLAG="/etc/mol/MOLTYPE"
 
 PATH="/sbin:/bin:/usr/sbin:/usr/bin"
 PATH="${PATH}:/BASE/sbin:/BASE/bin:/BASE/usr/sbin:/BASE/usr/bin"
@@ -394,9 +394,9 @@ checkbootparam "nonet" && { NONETWORK="yes"; }
 checkbootparam "novideo" && { NOVIDEO="yes"; }
 checkbootparam "nodhcp" && { NODHCP="yes"; }
 checkbootparam "ipaddr" && { IPADDR="$(getbootparam ipaddr 2>/dev/null)"; }
-checkbootparam "netmask" && { NETMASK="$(getbootparam ipaddr 2>/dev/null)"; }
-checkbootparam "gateway" && { GATEWAY="$(getbootparam ipaddr 2>/dev/null)"; }
-checkbootparam "netdev" && { NETDEV="$(getbootparam ipaddr 2>/dev/null)"; }
+checkbootparam "netmask" && { NETMASK="$(getbootparam netmask 2>/dev/null)"; }
+checkbootparam "gateway" && { GATEWAY="$(getbootparam gateway 2>/dev/null)"; }
+checkbootparam "netdev" && { NETDEV="$(getbootparam netdev 2>/dev/null)"; }
 # if not eject when halt
 checkbootparam "noeject" && { NOEJECT="yes"; }
 
@@ -585,8 +585,9 @@ do
 	then
 		if [ -f "/BASE${BASE_FLAG}" ]
 		then
-			MOL_TYPE="$(cat /BASE${BASE_FLAG})"
-			echo -n "${CRE} ${GREEN}Accessing Make One Linux ${MOL_TYPE} Filesystem at ${MAGENTA}${d}${GREEN}...${NORMAL}"
+			#MOL_TYPE="$(cat /BASE${BASE_FLAG})"
+			. /BASE${BASE_FLAG}
+			echo -n "${CRE} ${GREEN}Accessing Make One Linux ${MOLVERSION} ${MOL_TYPE} Filesystem at ${MAGENTA}${d}${GREEN}...${NORMAL}"
 			break
 		fi
 		umount /BASE
@@ -608,7 +609,6 @@ BASE_FS="${FOUND_FS}"
 ## set toram/tohd system
 if [ -n "${COPYTO}" ]
 then
-	echo
 	echo -n "${CRE} ${GREEN}Copying SYSTEM to "
 
 	mkdir /system
@@ -630,6 +630,7 @@ then
 		BASE_DEV="${COPYTOHD}"
 		BASE_FS="${FOUND_FS}"
 	fi
+	echo
 	# mount copyto device?
 	if [ -z "${FLAG}" ]
 	then
@@ -861,13 +862,21 @@ then
 	echo "none /UNIONFS unionfs rw,dirs=/MOL+=rw:${sysdir}=ro,debug=0,err=tryleft,delete=all,copyup=preserve,setattr=left 0 0" >> /etc/mtab.in
 	#echo "none /UNIONFS unionfs rw,dirs=/MOL+=rw:${sysdir}=ro,debug=0,err=tryleft,delete=all,copyup=preserve,setattr=left 0 0" >> /etc/mtab
 
+	# for yum bug
+	if [ -d /UNIONFS/var/cache/yum ]; then
+		rm -rf /UNIONFS/var/cache/yum
+		rm -f /MOL+/var/cache/.wh.yum
+		mkdir -p /MOL+/yum
+		ln -sf /MOL+/yum /MOL+/var/cache/yum
+	fi
 	# creating some directories which was removed when MOL system created
 	mkdir -p /UNIONFS/tmp /UNIONFS/var/tmp
 	ln -snf ../var/tmp /UNIONFS/usr/tmp
 	# change original files to this
 	cp -a /etc/rc.d/rc.sysinit /UNIONFS/etc/rc.d/
 	cp -a /etc/rc.d/init.d/halt /UNIONFS/etc/rc.d/init.d/
-	cp -a /etc/rc.d/init.d/netfs /UNIONFS/etc/rc.d/init.d/
+	[ -f /etc/rc.d/init.d/netfs ] && cp -a /etc/rc.d/init.d/netfs /UNIONFS/etc/rc.d/init.d/
+	[ -f /etc/rc.d/init.d/functions ] && cp -a /etc/rc.d/init.d/functions /UNIONFS/etc/rc.d/init.d/
 	cp -a /etc/fstab /UNIONFS/etc/
 	cp -a /etc/mtab /UNIONFS/etc/
 	# change original file to this after finishing linuxrc script
@@ -1005,7 +1014,7 @@ then
 	do
 		for conf in floppy mol mouse netcard sound xserver
 		do
-			[ "${f}" == "${conf#/etc/sysconfig/mol/}" ] && mv -f /etc/sysconfig/mol/${f} /etc/sysconfig/mol/${f}.backup
+			[ "${f#/etc/sysconfig/mol/}" == "${conf}" ] && mv -f /etc/sysconfig/mol/${f} /etc/sysconfig/mol/${f}.backup
 		done
 	done
 else
@@ -1140,41 +1149,77 @@ echo "6" > /proc/sys/kernel/printk
 ##< Make Permanent Home Directory >##
 
 ##< Make function of ejecting disk when halt >##
-if [ -z "${NOEJECT}" -a "${MODE}" != "overlay" ]; then
+if [ -z "${NOEJECT}" ]; then
+	RWDIR="/MOL+"
+	[ "${MODE}" == "overlay" ] && RWDIR="/UNIONFS"
 	ln -snf /MOL/lib /lib
-	rm -f /MOL+/lib
-	mkdir -p /MOL+/lib
-	ln -sn /MOL/lib/* /MOL+/lib/ 2>/dev/null
-	for lib in $(ldd /sbin/init | awk '{if($(NF-1)!="dynamic"){print $(NF-1)}}'); do
-		BASENAME="$(basename ${lib})"
-		BASEDIR="${lib%${BASENAME}}"
-		[ -z "${BASENAME}" -o -z "${BASEDIR}" ] && continue
-		[ -L /MOL+/${BASEDIR%/} ] && rm -f /MOL+${BASEDIR%/}
-		mkdir -p /MOL+${BASEDIR}
-		[ -f /MOL+${BASEDIR}${BASENAME} ] && rm -f /MOL+${BASEDIR}${BASENAME}
-		cp -a /MOL${BASEDIR}${BASENAME} /MOL+${BASEDIR}
+	if [ ! -d "${RWDIR}/lib" ]; then
+		rm -rf ${RWDIR}/lib 
+		[ "${MODE}" == "overlay" ] && rm -f /MOL+/.wh.lib # 0.6.2
+		mkdir -p ${RWDIR}/lib
+	fi
+	ln -sn /MOL/lib/* ${RWDIR}/lib/ 2>/dev/null
+	for lib in $(ldd /exec/init | awk '{if($(NF-1)!="dynamic"){print $(NF-1)}}'); do
+		[ -e "${RODIR}${lib}" ] || continue
+		#RWDIR="/MOL+"
+		RODIR="${sysdir}"
 	
-		if [ -L "${lib}" ]; then
-			LINKNAME="$(readlink ${lib})"
-			[ -z "${LINKNAME}" ] && continue
-			[ -f /MOL+${BASEDIR}${LINKNAME} ] && rm -f /MOL+${BASEDIR}${LINKNAME}
-			cp -a /MOL${BASEDIR}${LINKNAME} /MOL+${BASEDIR}
-		fi
+		## start
+		cd "${RWDIR}"
+		CURRENT_TARGET="${RODIR}"
+		current=""
 	
-		ln -sn /MOL${BASEDIR}* /MOL+${BASEDIR} 2>/dev/null
+		for current in $(echo "${lib}" | sed -e 's@/@ @g'); do
+			# for target
+			if [ "${current}" == "${lib##*/}" ]; then
+				rm -rf "${current}"
+				cp -a "${CURRENT_TARGET}/${current}" .
+				if [ -L "${current}" ]; then
+					LINKNAME="$(readlink ${current})"
+					[ -e "${LINKNAME}" ] && rm -rf "${LINKNAME}"
+					cp -a "${CURRENT_TARGET}/${LINKNAME}" ${LINKNAME}
+				fi
+				break
+			# for not exist
+			elif [ ! -e "${current}" ]; then
+				mkdir -p "${RWDIR}${lib}"
+			# for symlink
+			elif [ -L "${current}" ]; then
+				rm -f "${current}"
+				mkdir -p "${RWDIR}${lib}"
+			# for directory
+			elif [ -d "${current}" ]; then
+				:
+			# error
+			else
+				echo "Bad Destination."
+			fi
+			cd "${current}"
+			CURRENT_TARGET="${CURRENT_TARGET}/${current}"
+			ln -sn ${CURRENT_TARGET}/* . >/dev/null 2>&1
+		done
 	done
-	ln -snf /MOL+/lib /lib
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/rm -f /MOL+/lib
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/mkdir -p /MOL+/lib
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/mkdir -p /MOL+/lib/tls
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/cp -a /MOL/lib/libsepol.so.1 /MOL+/lib/
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/cp -a /MOL/lib/libselinux.so.1 /MOL+/lib/
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/cp -a /MOL/lib/tls/libc.so.6 /MOL+/lib/tls/
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/cp -a /MOL/lib/tls/libc-2.3.4.so /MOL+/lib/tls/
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/cp -a /MOL/lib/ld-linux.so.2 /MOL+/lib/
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/cp -a /MOL/lib/ld-2.3.4.so /MOL+/lib/
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/ln -sn /MOL/lib/* /MOL+/lib/
-	#/MOL/lib/ld-linux.so.2 --library-path /MOL/lib /MOL/bin/ln -sn /MOL/lib/tls/* /MOL+/lib/tls/
+#	for lib in $(ldd /exec/init | awk '{if($(NF-1)!="dynamic"){print $(NF-1)}}'); do
+#		BASENAME="$(basename ${lib})"
+#		BASEDIR="${lib%${BASENAME}}"
+#		[ -z "${BASENAME}" -o -z "${BASEDIR}" ] && continue
+#		[ -L "${RWDIR}${BASEDIR%/}" ] && rm -f ${RWDIR}${BASEDIR%/}
+#		mkdir -p ${RWDIR}${BASEDIR}
+#		[ -f "${RWDIR}${BASEDIR}${BASENAME}" ] && rm -f ${RWDIR}${BASEDIR}${BASENAME}
+#		cp -a /MOL${BASEDIR}${BASENAME} ${RWDIR}${BASEDIR}
+#	
+#		if [ -L "${lib}" ]; then
+#			LINKNAME="$(readlink ${lib})"
+#			[ -z "${LINKNAME}" ] && continue
+#			[ -f ${RWDIR}${BASEDIR}${LINKNAME} ] && rm -f ${RWDIR}${BASEDIR}${LINKNAME}
+#			cp -a /MOL${BASEDIR}${LINKNAME} ${RWDIR}${BASEDIR}
+#		fi
+#	
+#		ln -sn /MOL${BASEDIR}* ${RWDIR}${BASEDIR} 2>/dev/null
+#	done
+	ln -snf ${RWDIR}/lib /lib
+else
+	echo "NOEJECT=\"yes\"" >> /molinfo # temporary fix
 fi
 
 ##< Execute Final Script >##
